#ifndef ___MODBUS_H_20131014___
#define ___MODBUS_H_20121014___

#include <stdint.h>
#include <termios.h>

#define MODBUS_OK                                     0x00
#define MODBUS_ERROR_COMMUNICATION                    0x10
#define MODBUS_ERROR_COMMUNICATION_ILLEGAL_FUNCTION   0x11
#define MODBUS_ERROR_COMMUNICATION_ILLEGAL_ADDRESS    0x12
#define MODBUS_ERROR_COMMUNICATION_ILLEGAL_VALUE      0x13
#define MODBUS_ERROR_COMMUNICATION_ILLEGAL_OPERATION  0x14
#define MODBUS_ERROR_APPLICATION                      0x20
#define MODBUS_ERROR_UNKNOWN                          0x80

/**
 *  Modbus Read
 *
 *  * parameters
 *   int        fd:  file descripter for serial device
 *   uint8_t  addr:  slave device address
 *   uint16_t sr:    starting register number
 *   uint16_t nr:    number of registers to read
 *   uint8_t data[]: memory area for read data -- the size of footprint must be nr*2
 *
 *  * return values
 *    MODBUS_OK
 *    MODBUS_ERROR_COMMUNICATION
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_FUNCTION
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_ADDRESS
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_VALUE
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_OPERATION
 *    MODBUS_ERROR_APPLICATION
 *    MODBUS_ERROR_UNKNOWN
 */
int IEEE1888_ModbusRead(int fd, uint8_t addr, uint16_t sr, uint16_t nr, uint8_t data[]);

/**
 *  Modbus Write
 *
 *  * parameters
 *   int        fd:  file descripter for serial device
 *   uint8_t  addr:  slave device address
 *   uint16_t sr:    starting register number
 *   uint16_t nr:    number of registers to write
 *   uint8_t data[]: memory area for writing data -- the size of footprint must be nr*2
 *
 *  * return values
 *    MODBUS_OK
 *    MODBUS_ERROR_COMMUNICATION
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_FUNCTION
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_ADDRESS
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_VALUE
 *    MODBUS_ERROR_COMMUNICATION_ILLEGAL_OPERATION
 *    MODBUS_ERROR_APPLICATION
 *    MODBUS_ERROR_UNKNOWN
 */
int IEEE1888_ModbusWrite(int fd, uint8_t addr, uint16_t sr, uint16_t nr, uint8_t data[]);

int IEEE1888_OpenRS485(const char* device, speed_t baud);
void IEEE1888_CloseRS485(int fd);
#endif // #ifndef ___MODBUS_H_20131014___
