/*
 * server_basic_app.c  ieee1888 sdk server 使用示例
 * author: Lubia Yang
 * create: 2013-8-20
 * update: 2013-9-20
 */
#include<stdio.h>
#include<stdlib.h>
#include<time.h>
#include<string.h>
#include"server_basic.h"

#define IEEE1888_WRITE_SUCCESS 0
#define IEEE1888_WRITE_FAIL 1

/*
 * demo功能：对于每个查询的point,返回3条固定数据
 * server_query_app：一旦有查询请求发送到服务端且通过格式验证，将调用此函数
 * request中包含查询的id和类型，如查询最新值、最旧值或一段时间的数据，res_points中存放查询到的数据
 * 结构体fetch_points、points_value参见server_basic.h
 * ieee1888_mk_point_array(int n)：创建n个point,参见ieee1888.h,sdk负责GC
 * ieee1888_mk_value_array(int n)：对某个point,创建n个value,参见ieee1888.h,sdk负责GC
 * ieee1888_mk_string(const char* str)：创建字符串,sdk负责GC
 */
points_value* server_query_app(fetch_points* request) {
	/*
	points_value* res_points = (points_value*) calloc(sizeof(points_value), 1);
	res_points->n_point = request->n_key;//假设请求点都能查询到数据
	res_points->point = ieee1888_mk_point_array(res_points->n_point);//创建多个point
	int i = 0;
	for (i = 0; i < res_points->n_point; i++) {
		res_points->point[i].id = ieee1888_mk_string(request->key[i].id);//回复point的id必须等于请求point的id
		int j = 0;
		int n_value = 3;
		res_points->point[i].value = ieee1888_mk_value_array(n_value);//为point创建多个value
		res_points->point[i].n_value = n_value;
		for (j = 0; j < n_value; j++) {
			res_points->point[i].value[j].content = ieee1888_mk_string("10");//对value赋值
			res_points->point[i].value[j].time = ieee1888_mk_string("2013-08-14T16:00:00+08:00");//对value赋值
		}
	}
	return res_points;
	*/
	points_value* pv = IEEE1888_DataPoolFetch("data.txt",request);
	return pv;
}

/*
 * demo功能：打印写入的数据
 * server_data_app：一旦有写入请求发送到服务端且通过格式验证，将调用此函数
 * 操作成功返回IEEE1888_WRITE_SUCCESS，否则返回IEEE1888_WRITE_FAIL
 */
int server_data_app(points_value* request) {
	int i = 0;
	for (i = 0; i < request->n_point; i++) {
		int j = 0;
		printf("Point Id:%s\n",request->point[i].id);
		for (j = 0; j < request->point[i].n_value; j++) {
			printf("Time:%s\n",request->point[i].value[j].time);
			printf("Content:%s\n",request->point[i].value[j].content);
		}
	}
	IEEE1888_DataPoolWrite("data.txt",request);
	return IEEE1888_WRITE_SUCCESS;
}

/*
 * demo功能：打印错误信息
 * server_fault：一旦客户端请求引发服务端错误，则调用此函数
 * fault_code标记错误类型
 * fault_string标记错误的详细描述
 */
void server_fault_app(fault* f){
	printf("%s:",f->fault_code);
	printf("%s\n",f->fault_string);
}

/*
 * demo功能：用设计好的handler处理指定端口的数据
 * IEEE1888_ServerCreate(int n)：指定监听端口
 * 注意：监听的路径固定为IEEE1888GW，此demo监听路径为http://localhost:1888/IEEE1888GW
 */
int main() {
	IEEE1888_SetServerHandler(server_query_app, server_data_app,server_fault_app);
	int res = IEEE1888_ServerCreate(1888);
}
